/*------------------------------------------------------*
 | Author: Maurizio Loreti, aka MLO or (HAM) I3NOO      |
 | Work:   University of Padova - Department of Physics |
 |         Via F. Marzolo, 8 - 35131 PADOVA - Italy     |
 | Phone:  +39 (049) 827-7216   FAX: +39 (049) 827-7102 |
 | EMail:  loreti@pd.infn.it                            |
 | WWW:    http://www.pd.infn.it/~loreti/mlo.html       |
 *------------------------------------------------------*

 A test program for the binary tree routines.

 $Id: testTree.c,v 1.1 2001/01/17 11:20:10 loreti Exp $

 *------------------------------------------------------*/

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include "tree.h"

#ifndef NUMBER
#define NUMBER 5
#endif

static int  compare(const item *, const item *);
static void delete(item *);
static void print(const item *);
static void test(const item *);

static int nData;

int main() {
  int   saveData[NUMBER];
  node *root = 0;
  node *pNode;
  item  it;
  item *pItem;
  int   i;

/**
 | Insert NUMBER different random numbers in the binary tree, check if
 | they are there and check the ascending order.
**/

  for (i = 0;   i < NUMBER;   ++i) {
    pItem = malloc(sizeof(item));
    assert(pItem != 0);

    do {
       saveData[i] = pItem->data = rand();
    } while (insertTree(&root, pItem, &pNode, compare) != OK);

    it.data = saveData[i];
    assert(findTree(root, &it, compare) != 0);
  }
  nData = 0;
  traverseTree(root, INORDER, test);

/**
 | Remove every third number from the tree, check that the removed
 | items are missing, and check the ascending order.
**/

  for (i = 0;   i < NUMBER;   i += 3) {
    it.data = saveData[i];
    assert(removeFromTree(&root, &it, compare, delete) == OK);
    assert(findTree(root, &it, compare) == 0);
  }
  nData = 0;
  traverseTree(root, INORDER, test);

/**
 | Reinsert the removed numbers, check that they are there, and check
 | the ascending order.
**/

  for (i = 0;   i < NUMBER;   i += 3) {
    pItem = malloc(sizeof(item));
    assert(pItem != 0);
    pItem->data = it.data = saveData[i];
    assert(insertTree(&root, pItem, &pNode, compare) == OK);
    assert(findTree(root, &it, compare) != 0);
  }
  nData = 0;
  traverseTree(root, INORDER, test);

/**
 | Print the tree
**/

  nData = 0;
  traverseTree(root, INORDER, print);

/**
 | Remove the tree, and exit
**/

  removeTree(root, delete);
  return 0;
}

static int compare(
  const item *p1,
  const item *p2
) {
  return p1->data - p2->data;
}

static void delete(
  item *pI
) {
  free(pI);
}

static void print(
  const item *pI
) {
  printf("%d: %d\n", ++nData, pI->data);
}

static void test(
  const item *pI
) {
  static int oldData;

  if (nData++ != 0) assert(pI->data > oldData);
  oldData = pI->data;
}
